package api

import (
	"box-cost/db/model"
	"fmt"
	"regexp"

	_ "image/gif"
	_ "image/jpeg"
	_ "image/png"

	"github.com/xuri/excelize/v2"
)

type ProcessBillExcel struct {
	Offset int
	Row    int

	Title string //标题

	Excel *excelize.File

	SheetName string

	AlignCenterStyle int

	Content *model.PurchaseBill

	Signatures []*model.Signature
}

func (b *ProcessBillExcel) drawTitle() error {
	marginLeft := excelize.PageMarginLeft(0.3)
	b.Excel.SetPageMargins(b.SheetName, marginLeft)
	tileIndex := b.Offset + 1
	startCell := fmt.Sprintf("A%d", tileIndex)
	err := b.Excel.MergeCell(b.SheetName, startCell, fmt.Sprintf("J%d", tileIndex))
	if err != nil {
		return err
	}

	style, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Font:      &excelize.Font{Bold: true, Size: 18}})
	if err != nil {
		return err
	}
	err = b.Excel.SetCellStyle(b.SheetName, startCell, startCell, style)
	if err != nil {
		return err
	}
	b.Excel.SetRowHeight(b.SheetName, tileIndex, 23)
	b.Excel.SetCellValue(b.SheetName, startCell, b.Title)
	return nil
}

func (b *ProcessBillExcel) drawSubTitles() error {
	row := b.Offset + 2
	styleLeft, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}
	styleRight, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "right", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}

	var drawLeft = func(rowIndex int, value string) error {
		//左边1
		left1Cell := fmt.Sprintf("A%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, left1Cell, fmt.Sprintf("E%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, styleLeft)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, left1Cell, value)
		return nil
	}

	var drawRight = func(rowIndex int, value string) error {
		right1Cell := fmt.Sprintf("F%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, right1Cell, fmt.Sprintf("J%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, right1Cell, right1Cell, styleRight)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, right1Cell, value)
		return nil
	}
	//第一行
	drawLeft(row, "类别："+b.Content.Type)
	drawRight(row, "单号："+b.Content.SerialNumber)
	b.Excel.SetRowHeight(b.SheetName, row, 21)

	//第二行
	drawLeft(row+1, "供应商名称："+b.Content.Supplier)
	timeformat := b.Content.CreateTime.Local().Format("2006年01月02号 15:04:05")
	drawRight(row+1, "下单时间："+timeformat)
	b.Excel.SetRowHeight(b.SheetName, row+1, 21)

	//第三行
	drawLeft(row+2, "产品名称："+b.Content.ProductName)
	status := ""
	if b.Content.Status == "complete" {
		status = fmt.Sprintf("已完成(%d)", b.Content.ConfirmCount)
	}
	if b.Content.Status == "created" {
		status = "进行中"
	}
	drawRight(row+2, "状态："+status)
	b.Excel.SetRowHeight(b.SheetName, row+2, 21)

	return nil
}

func (b *ProcessBillExcel) drawTableTitle() error {
	row := b.Offset + 5
	// 品名 规格  数量 单位 单价 金额
	var drawCol = func(prefix string, value string) error {
		left1Cell := fmt.Sprintf("%s%d", prefix, row)
		left2Cell := fmt.Sprintf("%s%d", prefix, row+1)

		err := b.Excel.MergeCell(b.SheetName, left1Cell, left2Cell)
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left2Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}

		return b.Excel.SetCellValue(b.SheetName, left1Cell, value)
	}
	// a采购项目 b规格 c下单数量 d单位 e完成数量 f单价 g预算金额 h实际金额 i交货时间 j备注
	drawCol("A", "采购项目")
	drawCol("B", "规格")
	drawCol("C", "下单数量")
	drawCol("D", "单位")
	drawCol("E", "完成数量")
	drawCol("F", "单价")
	drawCol("G", "预算金额")
	drawCol("H", "实际金额")
	drawCol("I", "交货时间")
	drawCol("J", "备注")

	return nil
}

func (b *ProcessBillExcel) drawTableContent() error {
	row := b.Offset + 7
	b.Row = row

	var DrawRow = func(rowIndex int, values ...string) {
		charas := []string{"A", "B", "C", "D", "E", "F", "G", "H", "I", "J"}
		for i, c := range charas {
			v := ""
			if i < len(values) {
				v = values[i]
			}
			b.Excel.SetCellValue(b.SheetName, fmt.Sprintf("%s%d", c, rowIndex), v)
			val2Cel := fmt.Sprintf("%s%d", c, rowIndex)
			b.Excel.SetCellStyle(b.SheetName, val2Cel, val2Cel, b.AlignCenterStyle)

			b.Excel.SetRowHeight(b.SheetName, rowIndex, 21)
		}
	}
	ps := b.Content.Process
	if ps != nil {
		deliveryTime := ps.DeliveryTime.Local().Format("2006-01-02")
		confirmCount := "-"
		realAmount := "-"
		// 预算金额
		budgetAmount := fmt.Sprintf("%.3f", b.Content.BudgetAmount)
		b.FormatToEmpty(&budgetAmount)

		// 实际完成数
		confirmCount = fmt.Sprintf("%d", ps.ConfirmCount)
		b.FormatToEmpty(&confirmCount)

		// 实际金额
		realAmount = fmt.Sprintf("%.3f", b.Content.RealAmount)
		b.FormatToEmpty(&realAmount)

		// a采购项目 b规格 c下单数量 d单位 e完成数量 f单价 g预算金额 h实际金额 i交货时间 j备注
		orderCount := fmt.Sprintf("%d", ps.OrderCount)
		price := fmt.Sprintf("%.3f", ps.Price)
		b.FormatToEmpty(&price)
		DrawRow(row, ps.Name, ps.Norm, orderCount, ps.Unit, confirmCount, price, budgetAmount, realAmount, deliveryTime, ps.Remark)
		row++
		b.Row++
	}

	return nil
}

func (b *ProcessBillExcel) drawTableFooter() error {
	left1Cell := fmt.Sprintf("A%d", b.Row)
	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}
	styleLeft, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Border:    border,
	})

	b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, styleLeft)
	b.Excel.SetCellValue(b.SheetName, left1Cell, "送货地址：")

	addCel := fmt.Sprintf("B%d", b.Row)

	b.Excel.MergeCell(b.SheetName, addCel, fmt.Sprintf("E%d", b.Row))
	b.Excel.SetCellStyle(b.SheetName, addCel, fmt.Sprintf("E%d", b.Row), styleLeft)
	b.Excel.SetCellValue(b.SheetName, addCel, b.Content.SendTo)

	sureCel := fmt.Sprintf("F%d", b.Row)
	b.Excel.MergeCell(b.SheetName, sureCel, fmt.Sprintf("J%d", b.Row))
	b.Excel.SetCellStyle(b.SheetName, sureCel, fmt.Sprintf("J%d", b.Row), styleLeft)
	b.Excel.SetCellValue(b.SheetName, sureCel, "供应商签字：")

	b.Excel.SetRowHeight(b.SheetName, b.Row, 30)

	return nil
}

func (b *ProcessBillExcel) drawRemark() error {
	// 填备注
	b.Row++
	remarkTitleCell := fmt.Sprintf("A%d", b.Row)
	b.Excel.SetCellValue(b.SheetName, remarkTitleCell, "备注：")
	b.Row++

	remarkContentScell := fmt.Sprintf("A%d", b.Row)
	// 预留五行备注内容
	reg := regexp.MustCompile(`\n`)
	remarkRowNum := len(reg.FindAllString(b.Content.Remark, -1)) + 1
	b.Row += remarkRowNum
	remarkContentEcell := fmt.Sprintf("J%d", b.Row)
	b.Excel.MergeCell(b.SheetName, remarkContentScell, remarkContentEcell)
	b.Excel.SetCellValue(b.SheetName, remarkContentScell, b.Content.Remark)

	return nil

}

func (b *ProcessBillExcel) drawTableSignature() error {
	b.Row += 2
	style1, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		// Border:    border,
	})

	// 制单人
	billUserCell := fmt.Sprintf("A%d", b.Row+1)
	b.Excel.SetCellValue(b.SheetName, billUserCell, "制单人：")
	billUservCell := fmt.Sprintf("B%d", b.Row+1)
	b.Excel.SetCellValue(b.SheetName, billUservCell, b.Content.UserName)

	fontCell := fmt.Sprintf("F%d", b.Row)
	imageCell1 := fmt.Sprintf("G%d", b.Row)
	imageCell2 := fmt.Sprintf("I%d", b.Row)
	eRow := b.Row + 2
	b.Excel.MergeCell(b.SheetName, fontCell, fmt.Sprintf("F%d", eRow))
	b.Excel.SetCellStyle(b.SheetName, fontCell, fontCell, style1)
	b.Excel.SetCellValue(b.SheetName, fontCell, "审核签字：")

	b.Excel.MergeCell(b.SheetName, imageCell1, fmt.Sprintf("H%d", eRow))
	b.Excel.SetCellStyle(b.SheetName, imageCell1, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell1, "")

	b.Excel.MergeCell(b.SheetName, imageCell2, fmt.Sprintf("J%d", eRow))
	b.Excel.SetCellStyle(b.SheetName, imageCell2, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell2, "")

	// 状态为已审核时，签字
	// `{
	// 	"x_scale": 0.12,
	// 	"y_scale": 0.12,
	// 	"x_offset": 30,
	// 	"print_obj": true,
	// 	"lock_aspect_ratio": false,
	// 	"locked": false,
	// 	"positioning": "oncell"
	// }`
	if b.Content.Reviewed == 1 {
		imageCells := []string{imageCell1, imageCell2}
		if len(b.Signatures) > 0 {
			for k, sign := range b.Signatures {
				b.Excel.AddPicture(b.SheetName, imageCells[k], sign.Path, sign.Format)

			}
		}
	}

	return nil
}

func (b *ProcessBillExcel) Draws() {
	b.drawTitle()
	b.drawSubTitles()
	b.drawTableTitle()
	b.drawTableContent()
	b.drawTableFooter()
	b.drawRemark()
	b.drawTableSignature()
}

func NewProcessBill(f *excelize.File) *ProcessBillExcel {

	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	styleLeft, _ := f.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Border:    border,
		Font:      &excelize.Font{Size: 10},
	})

	b := &ProcessBillExcel{
		Title:            "原材料采购单",
		SheetName:        "Sheet1",
		Excel:            f,
		Offset:           0,
		AlignCenterStyle: styleLeft,
		Signatures:       make([]*model.Signature, 0),
	}

	f.SetColWidth(b.SheetName, "A", "J", 11.5)
	f.SetPageMargins(b.SheetName, excelize.PageMarginTop(0), excelize.PageMarginLeft(0), excelize.PageMarginRight(0))
	return b
}

func (b *ProcessBillExcel) FormatToEmpty(str *string) {
	if *str == "0" || *str == "0.000" {
		*str = ""
	}

}

func (b *ProcessBillExcel) PrintPurchType() string {
	return "process"
}

func (b *ProcessBillExcel) SetContent(content *model.PurchaseBill) {
	b.Content = content

}
func (b *ProcessBillExcel) SetTitle(title string) {
	b.Title = title

}
func (b *ProcessBillExcel) SetOffset(offset int) {
	b.Offset = offset

}
func (b *ProcessBillExcel) SetSignatures(sign []*model.Signature) {
	b.Signatures = sign

}
